/**************************************************************************
 *
 * Copyright 2016 Samsung Electronics co., Ltd. All Rights Reserved.
 *
 * Contact: Konstantin Drabeniuk <k.drabeniuk@samsung.com>
 * Contact: Andrii Sokolenko <a.sokolenko@samsung.com>
 * Contact: Roman Marchenko <r.marchenko@samsung.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
**************************************************************************/

#include "tc_tbm.h"

void TBMEnv::SetUp(void)
{
	setenv("XDG_RUNTIME_DIR", "/run", 1);
	setenv("TBM_DISPLAY_SERVER", "1", 1);
#if 0
	const char *test_backend;

	tdm_config_set_int(TBM_CONFIG_KEY_GENERAL_THREAD, ::testing::get<0>(GetParam()));
	tdm_config_set_int(TBM_CONFIG_KEY_GENERAL_COMMIT_PER_VBLANK, ::testing::get<1>(GetParam()));
	if (getenv("TBM_DEBUG_MODULE"))
		tdm_config_set_string(TBM_CONFIG_KEY_DEBUG_MODULE, "buffer,thread,event,vblank,commit,pp,capture");

	test_backend = ::testing::get<2>(GetParam());
	if (!test_backend)
		test_backend = TBM_DEFAULT_MODULE;
	tdm_config_set_string(TBM_CONFIG_KEY_GENERAL_BACKENDS, test_backend);
#endif
}

void TBMEnv::TearDown(void)
{
	unsetenv("XDG_RUNTIME_DIR");
	unsetenv("TBM_DISPLAY_SERVER");
}

/* tbm_bufmgr_init(), tbm_bufmgr_deinit() */

TEST_F(TBMEnv, BufmgrInitDeinit)
{
	tbm_bufmgr bufmgr;

	bufmgr = tbm_bufmgr_init(-1);
	EXPECT_NE(bufmgr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bufmgr_deinit(bufmgr);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

TEST_F(TBMEnv, BufmgrInitFewTimes)
{
	tbm_bufmgr bufmgr[10];
	int i;

	for (i = 0; i < 10; i++) {
		bufmgr[i] = tbm_bufmgr_init(-1);
		EXPECT_NE(bufmgr[i], nullptr);
		EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	}

	for (i = 0; i < 10; i++)
		tbm_bufmgr_deinit(bufmgr[i]);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

TEST_F(TBMEnv, BufmgrDeinitWithNULL)
{
	tbm_bufmgr_deinit(NULL);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

/* tbm_bufmgr_server_init() */
TEST_F(TBMEnv, BufmgrServerInitDeinit)
{
	tbm_bufmgr bufmgr;

	bufmgr = tbm_bufmgr_server_init();
	EXPECT_NE(bufmgr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bufmgr_deinit(bufmgr);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

TEST_F(TBMEnv, BufmgrServerInitWithoutEnv)
{
	tbm_bufmgr bufmgr;

	TBMEnv::TearDown();

	bufmgr = tbm_bufmgr_server_init();
	EXPECT_NE(bufmgr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	tbm_bufmgr_deinit(bufmgr);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

TEST_F(TBMEnv, BufmgrServerInitFewTimes)
{
	tbm_bufmgr bufmgr[10];
	int i;

	for (i = 0; i < 10; i++) {
		bufmgr[i] = tbm_bufmgr_server_init();
		EXPECT_NE(bufmgr[i], nullptr);
		EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	}

	for (i = 0; i < 10; i++)
		tbm_bufmgr_deinit(bufmgr[i]);

	ASSERT_TRUE(tbm_bufmgr_debug_get_ref_count() == 0);
}

/* int tbm_surface_internal_query_supported_formats(uint32_t **formats, uint32_t *num) */
TEST_F(TBMEnv, SurfaceInternalQueryFormats)
{
	tbm_bufmgr bufmgr;
	uint32_t num = 0;
	tbm_format *formats = NULL;

	bufmgr = tbm_bufmgr_server_init();
	EXPECT_NE(bufmgr, nullptr);

	EXPECT_EQ(tbm_surface_internal_query_supported_formats(&formats, &num), 1);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));
	EXPECT_GT(num, 0);
	EXPECT_NE(formats, nullptr);
	for (uint32_t i = 0; i < num; i++)
		EXPECT_NE(formats[i], 0);

	if (formats)
		free(formats);

	tbm_bufmgr_deinit(bufmgr);
}

TEST_F(TBMEnv, SurfaceInternalQueryFormatsFailNull)
{
	tbm_bufmgr bufmgr;
	uint32_t num = 0;
	tbm_format *formats = NULL;

	bufmgr = tbm_bufmgr_server_init();
	EXPECT_NE(bufmgr, nullptr);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_NONE));

	EXPECT_EQ(tbm_surface_internal_query_supported_formats(&formats, NULL), 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
	EXPECT_EQ(tbm_surface_internal_query_supported_formats(NULL, &num), 0);
	EXPECT_STREQ(tbm_error_str(tbm_get_last_error()), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	tbm_bufmgr_deinit(bufmgr);
}

TEST_F(TBMEnv, SurfaceQueryFormatsFailNull)
{
	uint32_t num = 0;
	tbm_format *formats = NULL;
	tbm_surface_error_e result = TBM_SURFACE_ERROR_NONE;

	result = (tbm_surface_error_e)tbm_surface_query_formats(&formats, NULL);
	EXPECT_STREQ(tbm_error_str(result), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));

	result = (tbm_surface_error_e)tbm_surface_query_formats(NULL, &num);
	EXPECT_STREQ(tbm_error_str(result), tbm_error_str(TBM_ERROR_INVALID_PARAMETER));
}

TEST_F(TBMEnv, SurfaceQueryFormatSuccess)
{
	uint32_t num = 0;
	tbm_format *formats = NULL;
	tbm_surface_error_e result = TBM_SURFACE_ERROR_NONE;

	result = (tbm_surface_error_e)tbm_surface_query_formats(&formats, &num);
	EXPECT_EQ(TBM_SURFACE_ERROR_NONE, result);
	EXPECT_GT(num, 0);
	EXPECT_NE((tbm_format *)NULL, formats);
	for (uint32_t i = 0; i < num; i++)
		EXPECT_NE(0, formats[i]);
	free(formats);
}

INSTANTIATE_TEST_CASE_P(TBMEnvParams,
						TBMEnv,
						Combine(Bool(), Bool(), Values("none")));
